import csv
from django.core.mail import EmailMultiAlternatives, get_connection
from django.contrib import admin
from django.http import HttpResponse
from .utils import get_register_model, get_register_model_admin
from .models import Newsletter


class RegisterAdmin(admin.ModelAdmin):
    fields = ['email', 'name', 'created_on', 'modified_on']
    readonly_fields = ('created_on', 'modified_on')
    list_display = ('email', 'created_on')
    list_filter = ['created_on']
    search_fields = ['name', 'email']
    actions = ['export_to_csv']

    def export_to_csv(self, request, queryset):
        """
        Action to export leads to CSV
        """

        response = HttpResponse(mimetype='text/csv')
        response['Content-Disposition'] = 'attachment; filename=leads.csv'
        writer = csv.writer(response)
        field_names = [field for field in self.fields]
        writer.writerow(field_names)
        for obj in queryset:
            writer.writerow([getattr(obj, field, '') for field in field_names])
        return response

    export_to_csv.short_description = "Export selected objects as csv file"


admin.site.register(get_register_model(), get_register_model_admin())


class NewsletterAdmin(admin.ModelAdmin):
    fields = ['subject', 'from_name', 'from_address', 'html_file']
    readonly_fields = ('created_on', 'modified_on')
    list_display = ('subject', 'created_on',)
    list_filter = ['created_on']
    search_fields = ['subject']
    actions = ['send']

    def send(self, request, queryset):
        """
        Action to send the current newsletter to all registered users
        """
        num_sent_emails = 0
        connection = get_connection()

        for obj in queryset:
            messages = []
            body = obj.html_file.read()
            msg = EmailMultiAlternatives(subject=obj.subject,
                                         body='',
                                         from_email=u"{from_name} <{from_email}>".format(from_name=obj.from_name,
                                                                                         from_email=obj.from_address))
            msg.attach_alternative(body, "text/html")
            for obj in get_register_model().objects.all():
                msg.to = obj.email
                messages.append(msg)

            num_sent_emails += connection.send_messages(messages)

        return num_sent_emails

    send.short_description = "Send newsletter to all registered users"


admin.site.register(Newsletter, NewsletterAdmin)



from importlib import import_module

from django.conf import settings


def get_class_from_string(import_string):

    try:
        from_list, import_name = import_string.rsplit('.', 1)
        return getattr(import_module(from_list), import_name)
    except (AttributeError, ValueError):  # pragma no cover
        raise ImportError(import_string)


def get_register_model():
    return get_class_from_string(getattr(settings, 'LEADS_REGISTER_MODEL', 'leads.models.Register'))


def get_register_model_admin():
    return get_class_from_string(getattr(settings, 'LEADS_REGISTER_MODEL_ADMIN', 'leads.admin.RegisterAdmin'))


def get_register_form_class():
    return get_class_from_string(getattr(settings, 'LEADS_REGISTER_FORM_CLASS', 'leads.forms.RegisterForm'))


def get_register_form_fields():
    return getattr(settings, 'LEADS_REGISTER_FORM_FIELDS', ('name', 'email'))



# -*- coding: utf-8 -*-
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit
from django.utils.translation import ugettext_lazy as _

import floppyforms as forms
from .utils import get_register_model, get_register_form_fields


class RegisterForm(forms.ModelForm):
    """
    RegisterForm is intended to be a highly customizable ModelForm and
    the user can change the model associated to the form and form fields
    """

    def __init__(self, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.add_input(Submit('submit', _('Submit')))
        super(RegisterForm, self).__init__(*args, **kwargs)

    class Meta:
        model = get_register_model()
        fields = get_register_form_fields()



# -*- coding: utf-8 -*-
from django.core.urlresolvers import reverse
from django.views.generic import CreateView, TemplateView
from .utils import get_register_model, get_register_form_class


class IndexView(CreateView):
    """
    This view renders the main page
    """
    template_name = 'leads/index.html'
    model = get_register_model()
    form_class = get_register_form_class()

    def get_success_url(self):
        return reverse('leads:thanks_register')


class ThanksView(TemplateView):
    """
    Page that loads when someone registers successfully
    """
    template_name = "leads/thanks_register.html"



from django.conf.urls import patterns, url


from .views import IndexView, ThanksView

urlpatterns = patterns(
    '',

    # Index lead page
    url(r'^$', IndexView.as_view(), name='index'),
    url(r'^thanks/$', ThanksView.as_view(), name='thanks_register')
)



# -*- coding: utf-8 -*-
from django.db import models
from django.db.models.signals import post_delete
from django.utils.translation import ugettext_lazy as _


class Register(models.Model):
    """
    Register model to gather information among the visitors of the lead page
    """
    email = models.EmailField(unique=True)
    name = models.CharField(_('Name'), max_length=50, blank=True)
    created_on = models.DateTimeField(_('Created on'), auto_now_add=True)
    modified_on = models.DateTimeField(_('Modified on'), auto_now=True)

    class Meta:
        verbose_name = _('Register')
        verbose_name_plural = _('Registers')

    def __str__(self):
        return self.email


def delete_files_handler(sender, instance, **kwargs):
    for field in instance._meta.fields:
        if isinstance(field, models.FileField):
            f = getattr(instance, field.name)
            if f:
                f.delete(save=False)


class Newsletter(models.Model):
    """
    Defines a newsletter to send to regsistered users
    """

    from_name = models.CharField(_('From name'), max_length=50)
    from_address = models.EmailField(_('From address'))
    subject = models.CharField(_('Subject'), max_length=255)
    html_file = models.FileField(_('HTML file'), upload_to='newsletters')
    created_on = models.DateTimeField(_('Created on'), auto_now_add=True)
    modified_on = models.DateTimeField(_('Modified on'), auto_now=True)

    def __str__(self):
        return self.subject


post_delete.connect(delete_files_handler, Newsletter)
